"""
Package with core functions for decompiling pyc-files.
"""

__all__ = ["orion_decompile_fupy_2", \
           "orion_decompile_uncompile2_2", \
           "orion_decompile_pyretic_2", \
           "orion_decompile_decompylepp_2"]

def orion_decompile_fupy_2(filename, output="", indent_count=4):
    from sys import stdout
    from os import path, makedirs
    from fupy.fupy import PythonDecompiler, PythonDecompilerError
    indent_pattern = " "
    if indent_count > 0:
        indent_pattern = indent_pattern * indent_count
    if output == "":
        outstream = stdout
    else:
        try:
            outdir = path.dirname(output)
            if not path.isfile(outdir) and not path.isdir(outdir):
                makedirs(outdir) 
            outstream = open(output, "w")  
        except IOError:
          print("Error: cannot open '%s' for writing!" % output)
          return
    try:
        pydec = PythonDecompiler(filename)
        dump = pydec.decompile(indent = indent_pattern)
        outstream.write(dump)
        outstream.write("\n")
    except PythonDecompilerError as e:
        print("Error: " + str(e))
    finally:
        outstream.close()

def orion_decompile_uncompile2_2(filename, output=""):
    from sys import version, stderr, stdout
    from os import path, makedirs
    if version[:3] != "2.7":
        print >>stderr, "Error: Uncompyle2 requires Python 2.7!"
        return
    from uncompyle2 import uncompyle_file
    if output == "":
        outstream = stdout
    else:
        outdir = path.dirname(output)
        if not path.isfile(outdir) and not path.isdir(outdir):
            makedirs(outdir)        
        outstream = open(output, "w")
    try:
        uncompyle_file(filename, outstream) 
        outstream.write("\n")
    except Exception as e:
        print("Error: "+str(e))
    finally:
        outstream.close()
        
def orion_decompile_pyretic_2(filename, output=""):
    from sys import stdout
    from os import path, makedirs
    from pyretic.liveUnPYC import liveUnPYC
    if output == "":
        outstream = stdout
    else:
        outdir = path.dirname(output)
        if not path.isfile(outdir) and not path.isdir(outdir):
            makedirs(outdir)
        outstream = open(output, "w")
    try:
        live = liveUnPYC(None, None, False, False)
        outstream.write(live.fs_decompile(filename))
        outstream.write("\n")
    except Exception as e:
        print("Error: "+str(e))
    finally:
        outstream.close()

def orion_decompile_decompylepp_2(filename, output=""):
    from sys import stdout
    from os import path, makedirs
    from shutil import rmtree
    from zipfile import ZipFile
    from tempfile import gettempdir
    import uuid, subprocess
    try:
        fn, _ = __file__.split(".zip")
        fh = open(fn + ".zip","rb")
        z = ZipFile(fh)
        nm = "dcpack/decompylepp/pycdc.exe"
        outpath = gettempdir()
        if outpath[-1] != "\\":
            outpath = outpath + "\\"
        outpath = outpath + str(uuid.uuid1()).replace("-", "") + "\\"
        z.extract(nm, outpath)
    except Exception as e:
        print("Error: "+str(e))
        return
    if output == "":
        outstream = stdout
    else:
        try:
            outdir = path.dirname(output)
            if not path.isfile(outdir) and not path.isdir(outdir):
                makedirs(outdir)            
            outstream = open(output, "w")
        except IOError:
          print("Error: cannot open '%s' for writing!" % output)
          rmtree(outpath,ignore_errors=True)
          return
    try:      
        outstream.write(subprocess.Popen([outpath+nm,filename],stdout=subprocess.PIPE).communicate()[0])
        outstream.write("\n")
    except Exception as e:
        print("Error: "+str(e))
    finally:
        outstream.close()
        rmtree(outpath,ignore_errors=True)